/**
 * This file registers Handlebars helper functions which can be used in Handlebars
 * HTML templates throughout the project.
 */

/**
 * The 'ifCond' helper is used to evaluate the two given operands, v1, and v2,
 * using the given boolean comparison operator passed in the operator field.
 * Return value is determined by the evaluation of the statement v1 operator v2,
 * returning options.fn(this) if true, or options.inverse(this) if false.
 */
Handlebars.registerHelper('ifCond', function (v1, operator, v2, options) {
    switch (operator) {
        case '==':
            return (v1 == v2) ? options.fn(this) : options.inverse(this);
        case '===':
            return (v1 === v2) ? options.fn(this) : options.inverse(this);
        case '<':
            return (v1 < v2) ? options.fn(this) : options.inverse(this);
        case '<=':
            return (v1 <= v2) ? options.fn(this) : options.inverse(this);
        case '>':
            return (v1 > v2) ? options.fn(this) : options.inverse(this);
        case '>=':
            return (v1 >= v2) ? options.fn(this) : options.inverse(this);
        case '&&':
            return (v1 && v2) ? options.fn(this) : options.inverse(this);
        case '||':
            return (v1 || v2) ? options.fn(this) : options.inverse(this);
        default:
            return options.inverse(this);
    }
});
